/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.runtime;

import java.util.HashMap;

public class Cflow extends ThreadLocal {
    public static Cflow singleton = new Cflow();

    public static class Entry {
        public int counter = 0;
    }

    protected synchronized Object initialValue() {
        return new HashMap();
    }

    public boolean check(String key) {
        return getEntry(key).counter > 0;
    }

    public void enter(String key) {
        getEntry(key).counter++;
    }

    public void exit(String key) {
        getEntry(key).counter--;
    }

    private Entry getEntry(String key) {
        HashMap hash = (HashMap)get();
        Entry e = (Entry)hash.get(key);
        if (e == null) {
            e = new Entry();
            hash.put(key, e);
        }

        return e;
    }
}
