/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import javassist.gluonj.pc.PointcutNode;

public class Advice {
    public static final String[] kinds = { "before", "after", "around" };
    public static final int BEFORE = 1;
    public static final int AFTER = 2;
    public static final int AROUND = 3;
    public static final char DOUBLE_QUOTE = '`';

    int kind;
    private PointcutNode pointcut;
    private AliasList aliases;
    private String advice;
    private String body;    // expanded advice body

    public Advice(int k, PointcutNode p, AliasList alist, String adv) {
        kind = k;
        pointcut = p;
        aliases = alist;
        advice = adv.replace(DOUBLE_QUOTE, '"');
        if (aliases != null)
            body = aliases.expand(advice);
        else
            body = advice;
    }

    public PointcutNode getPointcut() { return pointcut; }

    public String getBody() { return body; }

    public String toString() {
        return getKind() + AliasList.toString(aliases) + ": "
               + pointcut.toString() + " { " + advice + " }";
    }

    private String getKind() {
        if (kind < kinds.length)
            return kinds[kind];
        else
            return "unknown";
    }
}
