/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import java.util.HashMap;

public class AliasList {
    public AliasList next;
    private String alias;
    private String substExpr;

    public AliasList() {
        next = null;
        substExpr = alias = null;
    }

    public AliasList(String alias, String expr) {
        this.substExpr = expr;
        this.alias = alias;
        this.next = null;
    }

    public static AliasList concat(AliasList list1, AliasList list2) {
        AliasList a = list1;
        if (a == null)
            return list2;

        while (a.next != null)
            a = a.next;

        a.next = list2;
        return list1;
    }

    public static String toString(AliasList a) {
        if (a == null)
            return "[]";
        else
            return a.toString();
    }

    public String toString() {
        StringBuffer sbuf = new StringBuffer();
        sbuf.append('[');
        AliasList a = this;
        while (true) {
            sbuf.append(a.alias);
            sbuf.append('=');
            sbuf.append(a.substExpr);
            a = a.next;
            if (a == null)
                break;

            sbuf.append(',');
        }

        sbuf.append(']');
        return sbuf.toString();
    }

    public String expand(String src) {
        HashMap hash = new HashMap();
        expand(hash);
        return expand(src, hash);
    }

    private void expand(HashMap hash) {
        if (next != null)
            next.expand(hash);

        String expr = expand(substExpr, hash);
        hash.put(alias, "(" + expr + ")");
    }

    private static String expand(String src, HashMap hash) {
        StringBuffer dest = new StringBuffer();
        StringBuffer token = new StringBuffer();
        boolean start = true;
        int len = src.length();
        for (int i = 0; i < len; i++) {
            char c = src.charAt(i);
            if (start)
                if (Character.isJavaIdentifierStart(c)) {
                    start = false;
                    token.append(c);
                }
                else
                    dest.append(c);
            else
                if (Character.isJavaIdentifierPart(c))
                    token.append(c);
                else {
                    start = true;
                    appendToken(dest, token, hash);
                    token = new StringBuffer();
                    dest.append(c);
                }
        }

        if (token.length() > 0)
            appendToken(dest, token, hash);

        return dest.toString();
    }

    private static void appendToken(StringBuffer dest, StringBuffer token, HashMap hash) {
        String t = token.toString();
        String t2 = (String)hash.get(t);
        if (t2 != null)
            t = t2;

        dest.append(t);
    }
}
